# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

import datetime

from pprint import pprint

from steelscript.common.app import Application
from steelscript.common.timeutils import TimeParser
from steelscript.common.connection import Connection
from steelscript.common.exceptions import RvbdHTTPException

# Mapping from price measure to the relative position
# in the response string
mapping = {'open': 1,
           'high': 2,
           'low': 3,
           'close': 4,
           'volume': 5}

tp = TimeParser()


class StockApiException(Exception):
    pass


def parse_date(date):
    return tp.parse(date + " 00:00")


def get_historical_prices(begin, end, symbol, measures,
                          resolution='day', date_obj=False):
    """Get historical prices for the given ticker symbol.
    Returns a list of dicts keyed by 'date' and measures

    :param string begin: begin date of the inquire interval
      in the format of YYYY-MM-DD
    :param string end: end date of the inquire interval
      in the format of YYYY-MM-DD
    :param string symbol: symbol of one stock to query
    :param list measures: a list of prices that needs to be queried,
      should be a subset of ["open", "high", "low", "close", "volume"]
    :param string resolution: '1 day' or '5 days'
    :param boolean date_obj: dates are converted to datetime objects
      from date strings if True. Otherwise, dates are stored as strings
    """
    try:
        conn = Connection('http://ichart.finance.yahoo.com')
        start_month = parse_date(begin).month - 1
        start_day = parse_date(begin).day
        start_year = parse_date(begin).year
        end_month = parse_date(end).month - 1
        end_day = parse_date(end).day
        end_year = parse_date(end).year

        ret = []
        params = {'s': symbol,
                  'a': start_month,
                  'b': start_day,
                  'c': start_year,
                  'd': end_month,
                  'e': end_day,
                  'f': end_year,
                  'g': resolution[0],
                  'ignore': '.csv'}

        resp = conn.request(method='POST', path='/table.csv', params=params)
        # extract data and skip first row with column titles
        data = list(resp.iter_lines())[1:]

        # iterate over the data backwards as the daily prices are sorted
        # backwards by the dates
        for day in reversed(data):
            # day is a string with date, prices, volume separated by commas,
            # '<date>,<open>,<high>,<low>,<close>,<volume>,<adj_close>'
            # as '2014-02-19,20.22,20.55,20.11,20.50,1599600,20.50'
            day = day.split(',')
            date = parse_date(day[0]) if date_obj else day[0]
            daily_prices = {'date': date}
            for m in measures:
                if m in mapping:
                    daily_prices[m] = float(day[mapping[m]])
            ret.append(daily_prices)
    except RvbdHTTPException:
        raise StockApiException("Symbol '%s' is invalid or Stock '%s' was"
                                " not on market on %s" % (symbol, symbol,
                                                          end))
    return ret


class StockApp(Application):

    today = str(datetime.datetime.now().date())

    def add_options(self, parser):
        super(StockApp, self).add_options(parser)

        parser.add_option('-b', '--begin', help='begin date as YYYY-MM-DD')
        parser.add_option('-e', '--end', help='end date as YYYY-MM-DD',
                          default=self.today)

        parser.add_option('-r', '--resolution', default='day',
                          help='day or week')
        parser.add_option('-s', '--symbol', help='symbol of the stock')
        parser.add_option('-m', '--measures',
                          help=("list of measures to inquire, "
                                "such as open, high, low, close, volume, "
                                "delimited by commas"))

    def validate_args(self):
        super(StockApp, self).validate_args()

        if not self.options.symbol:
            self.parser.error("Symbol needs to be specified")

        if not self.options.measures:
            self.parser.error("Measures needs to be specified")
        else:
            measures = self.options.measures.split(',')
            for measure in measures:
                if measure not in ['open', 'high', 'low', 'close', 'volume']:
                    self.parser.error("Invalid measure %s" % measure)

        if self.options.resolution not in ['day', 'week']:
            self.parser.error("Invalid resolution %s" %
                              self.options.resolution)

        if not self.options.begin:
            self.parser.error("Begin date needs to be specified")
        else:
            try:
                begin_date = parse_date(self.options.begin)
            except:
                self.parser.error("Begin date %s is invalid" %
                                  self.options.begin)

        try:
            end_date = parse_date(self.options.end)
        except:
            self.parser.error("End date %s is invalid" % self.options.end)

        # begin date should be less than the both today and end date
        if begin_date > min(end_date, parse_date(self.today)):
            self.parser.error("Begin date %s is later than either "
                              "today's date %s or end date %s" %
                              (begin_date, self.today, end_date))

    def main(self):
        pprint(get_historical_prices(self.options.begin,
                                     self.options.end,
                                     self.options.symbol,
                                     self.options.measures.split(','),
                                     self.options.resolution))
if __name__ == '__main__':
    StockApp().run()



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

"""
This file defines a single report of multiple tables and widgets.

The typical structure is as follows:

    report = Report.create('Stock Report')
    report.add_section()

    table = SomeTable.create(name, table_options...)
    table.add_column(name, column_options...)
    table.add_column(name, column_options...)
    table.add_column(name, column_options...)

    report.add_widget(yui3.TimeSeriesWidget, table, name, width=12)

See the documeantion or sample plugin for more details
"""

from steelscript.appfwk.apps.report.models import Report
import steelscript.appfwk.apps.report.modules.yui3 as yui3

# Import the datasource module for this plugin (if needed)
import steelscript.stock.appfwk.datasources.stock_source as stock

report = Report.create("Stock Report-Multiple Stocks", position=11)

report.add_section()

#
# Define a stock table with current prices with a list of stocks
#

table = stock.MultiStockPriceTable.create(name='multi-stock-price',
                                          duration='52w', resolution='day',
                                          stock_symbol=None)

# Add columns for time and 3 stock columns
table.add_column('date', 'Date', datatype='date', iskey=True)

# Bind the table to a widget for display
report.add_widget(yui3.TimeSeriesWidget, table, 'Close Prices', width=12)


# Implement a widget to display the daily volumns for multiple stocks
table = stock.MultiStockVolumeTable.create(name='multi-stock-volume',
                                           duration='52w', resolution='day')
table.add_column('date', 'Date', datatype='date', iskey=True)
report.add_widget(yui3.TimeSeriesWidget, table, 'Daily Volumes', width=12,
                  bar=True)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

"""
This file defines a data source for querying data.

There are three parts to defining a data source:

* Defining column options via a Column class
* Defining table options via a DatasourceTable
* Defining the query mechanism via TableQuery

Note that you can define multiple Column and Table classes
in the same file, but only one TableQuery.  If you need
to define multiple types of queries, create multiple
files in this directory named accordingly.

"""

import logging
import pandas

from steelscript.appfwk.apps.datasource.models import \
    DatasourceTable, TableQueryBase, Column, TableField

from steelscript.appfwk.apps.datasource.forms import (
    fields_add_time_selection, fields_add_resolution,
    DateTimeField, ReportSplitDateWidget)

from steelscript.stock.core.app import get_historical_prices
from steelscript.appfwk.apps.jobs import \
    QueryComplete

logger = logging.getLogger(__name__)


class StockColumn(Column):
    class Meta:
        proxy = True

    COLUMN_OPTIONS = {}


class StockTable(DatasourceTable):
    class Meta:
        proxy = True

    _column_class = 'StockColumn'
    FIELD_OPTIONS = {'duration': '4w',
                     'durations': ('4w', '12w', '24w', '52w', '260w', '520w'),
                     'resolution': 'day',
                     'resolutions': ('day', 'week')
                     }

    def post_process_table(self, field_options):
        # Add a time selection field
        fields_add_time_selection(self, show_end=False,
                                  initial_duration=field_options['duration'],
                                  durations=field_options['durations'])

        # Add time resolution selection
        fields_add_resolution(self,
                              initial=field_options['resolution'],
                              resolutions=field_options['resolutions'])

        # Add end date field
        self.fields_add_end_date()

    def fields_add_stock_symbol(self, help_text, keyword='stock_symbol',
                                initial=None):
        field = TableField(keyword=keyword,
                           label='Stock Symbol',
                           help_text=(help_text),
                           initial=initial,
                           required=True)
        field.save()
        self.fields.add(field)

    def fields_add_end_date(self, initial_end_date='now-0'):
        # Add a date field
        # the front javascript code will determine the default date
        # according to initial_end_date, so if initial_end_date is
        # 'now-0', today will be the default end date
        field = TableField(keyword='end_date',
                           label='End Date',
                           field_cls=DateTimeField,
                           field_kwargs={'widget': ReportSplitDateWidget,
                                         'widget_attrs': {'initial_date':
                                                          initial_end_date}},
                           required=False)
        field.save()
        self.fields.add(field)


class SingleStockTable(StockTable):
    """Table class associated with report showing stock prices
    for a single stock, including open, high, low, close prices.
    """
    class Meta:
        proxy = True

    _query_class = 'SingleStockQuery'

    TABLE_OPTIONS = {'stock_symbol': None}

    def post_process_table(self, field_options):
        super(SingleStockTable, self).post_process_table(field_options)

        # Add stock symbol
        self.fields_add_stock_symbol('Single ticker symbol')


class MultiStockTable(StockTable):
    """Table class associated with report showing close prices
    for the given date range for input multiple stocks.
    """
    class Meta:
        proxy = True

    TABLE_OPTIONS = {'stock_symbol': None}

    def post_process_table(self, field_options):
        super(MultiStockTable, self).post_process_table(field_options)

        # Add stock symbol
        self.fields_add_stock_symbol('Ticker symbols separated by commas')


class MultiStockPriceTable(MultiStockTable):
    class Meta:
        proxy = True

    _query_class = 'MultiStockPriceQuery'


class MultiStockVolumeTable(MultiStockTable):
    class Meta:
        proxy = True

    _query_class = 'MultiStockVolumeQuery'


class StockQuery(TableQueryBase):

    def prepare(self):
        """Prepare data for query to run"""
        criteria = self.job.criteria

        # These are date time strings in the format of YYYY-MM-DD
        start_date = criteria.end_date - criteria.duration
        self.t0 = start_date.date().strftime('%Y-%m-%d')
        self.t1 = criteria.end_date.date().strftime('%Y-%m-%d')

        # Resolution is either day or week
        self.resolution = ('day' if str(criteria.resolution).
                           startswith('1 day') else 'week')

        # stock symbol string (can have multiple symbol)
        self.symbol = criteria.stock_symbol

        # Dict storing stock prices/volumes according to specific report
        self.data = None

    def get_data(self, symbol, measures, date_obj=False):
        data = get_historical_prices(self.t0, self.t1, symbol, measures,
                                     self.resolution, date_obj=date_obj)
        return pandas.DataFrame(data)


class SingleStockQuery(StockQuery):
    """Single stock query class used by candle stick widget
    to print graph that shows four prices of one stock for
    each day. Four prices include open, high, low, close prices.
    """
    def run(self):
        self.prepare()
        measures = ["open", "high", "low", "close"]
        df = self.get_data(self.symbol, measures)
        return QueryComplete(df)


class MultiStockQuery(StockQuery):
    """Base query class to fetch prices for multiple stocks"""
    def prepare(self):
        super(MultiStockQuery, self).prepare()

    def merge_price_history(self, his):
        """Merge history of another stock with self.data"""
        if self.data is None:
            self.data = his
        else:
            # As some stock might be off market on certain random days
            # thus merging the data frames need to be done according to date
            self.data = self.data.merge(his, on='date', how='outer')

    def run_query(self, measure=None):
        # delete non-key columns associated with table
        for c in self.table.get_columns():
            if not c.iskey:
                c.delete()

        for ticker in self.symbol.split(","):
            # strip white spaces
            ticker = ticker.strip().lower()
            if ticker not in map(lambda x: x.name, self.table.get_columns()):
                StockColumn.create(self.table, ticker, ticker.upper())
            if measure is None:
                measure = "close"
            history = self.get_data(ticker, [measure], date_obj=True)
            history.rename(columns={measure: ticker}, inplace=True)
            self.merge_price_history(history)
        return QueryComplete(self.data)


class MultiStockPriceQuery(MultiStockQuery):
    """Query to fetch one measure of stock prices for multiple stocks"""
    def run(self):
        super(MultiStockPriceQuery, self).prepare()
        return self.run_query()


class MultiStockVolumeQuery(MultiStockQuery):
    """Query to fetch daily volumes for multiple stocks"""
    def run(self):
        super(MultiStockVolumeQuery, self).prepare()
        return self.run_query("volume")



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

"""
This file defines custom models for this plugin

A custom model is a Django model - a Python class that is
backed by a database table.  This allows storage and retrieval
of custom data that is required by this plugin.

Many plugins do not require custom models and leave this file empty.

If models are defined, this plugin must be explicitly added
as an application in the local_settings.py file under LOCAL_APPS:

    LOCAL_APPS = (
        'steelscript.stock.appfwk',
    )

Often a related admin.py file is placed in this directory to provide
an interface for managing the model via the admin pages.

Sample model:

    class Product(models.Model):
        name = models.CharField(max_length=200)
        price = models.DecimalField(max_digits=7, decimal_places=3)
        count = models.IntegerField(default=1)

See https://docs.djangoproject.com/en/dev/ref/models/fields/ for a
full list of available models.

"""



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

"""This file defines a single Command"""

from optparse import OptionParser, OptionGroup
from steelscript.commands.steel import BaseCommand


class Command(BaseCommand):
    # Provide a one line help message
    help = 'Simple command'

    def add_options(self, parser):
        # Add OptionParser style options for your command
        # See https://docs.python.org/2/library/optparse.html

        # parser.add_option(
        #     '-b', '--beta', default=10,
        #     help='Show more detailed Python installation information')

        # You can also create option groups to provide
        # more organization for long commands
        #
        # group = OptionGroup(parser, 'Special options'
        # group.add_option(....)  -- just like add_option above
        # group.add_option(....)
        # parser.add_option_group(group)

    def main(self):
        # Main execution.  Options are available via self.options
        # print('Running with beta: {beta}'.format(beta=self.options.beta))



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

"""
This file defines a custom admin page for any models defined
in models.py.

A sample admin page for the sample Product model in models.py:

    from django.contrib import admin

    from steelscript.stock.appfwk.models import Product

    class ProductAdmin(admin.ModelAdmin):
        list_display = ('name', 'price', 'count')

    admin.site.register(Product, ProductAdmin)

"""



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.

"""
This file defines a single report of multiple tables and widgets.

The typical structure is as follows:

    report = Report.create('Stock Report')
    report.add_section()

    table = SomeTable.create(name, table_options...)
    table.add_column(name, column_options...)
    table.add_column(name, column_options...)
    table.add_column(name, column_options...)

    report.add_widget(yui3.TimeSeriesWidget, table, name, width=12)

See the documeantion or sample plugin for more details
"""
from steelscript.appfwk.apps.report.models import Report
from steelscript.appfwk.apps.datasource.models import Column
import steelscript.appfwk.apps.report.modules.yui3 as yui3

# Import the datasource module for this plugin (if needed)
import steelscript.stock.appfwk.datasources.stock_source as stock

report = Report.create("Stock Report-CandleStick", position=11)

report.add_section()

#
# Define a stock table with current prices with a list of stocks
#
table = stock.SingleStockTable.create(
    name='stock-table', duration='52w', resolution='day', stock_symbol=None)

# Add columns for time and 3 stock columns
table.add_column('date', 'Date', datatype=Column.DATATYPE_STRING, iskey=True)
table.add_column('open', 'Open')
table.add_column('high', 'High')
table.add_column('low', 'Low')
table.add_column('close', 'Close')

# Bind the table to a widget for display
report.add_widget(yui3.CandleStickWidget, table, 'Daily Prices', width=12)



# Copyright (c) 2015 Riverbed Technology, Inc.
#
# This software is licensed under the terms and conditions of the MIT License
# accompanying the software ("License").  This software is distributed "AS IS"
# as set forth in the License.


from setuptools import setup, find_packages
from gitpy_versioning import get_version

install_requires = (
    'steelscript.appfwk',
)

setup_args = {
    'name':                'steelscript.stock',
    'namespace_packages':  ['steelscript'],
    'version':             get_version(),
    'author':             'Riverbed Technology',
    'author_email':       'eng-github@riverbed.com',
    'url':                'http://pythonhosted.org/steelscript',
    'license':             'MIT',
    'description':         'Stock Report plugin for SteelScript Application Framework',
    'long_description':    """\
Stock Report for SteelScript Application Framework
==================================================

SteelScript is a collection of libraries and scripts in Python and JavaScript
for interacting with Riverbed Technology devices.

This package demonstrates how an external data source can be incorporated
as a plugin to App Framework, with associated reports.

For a complete guide to installation, see:

http://pythonhosted.org/steelscript/
    """,

    'packages': find_packages(exclude=('gitpy_versioning',)),
    'zip_safe': False,
    'install_requires': install_requires,
    'extras_require': None,
    'test_suite': '',
    'include_package_data': True,
    'entry_points': {
        'portal.plugins': [
            'stock = steelscript.stock.appfwk.plugin:Plugin'
        ],
    },

    'classifiers': (
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Topic :: Software Development'
    ),
}

setup(**setup_args)
